import { nextId, batch, isDefined } from '@collectable/core';
import { NONE } from './node';
import { unwrap, iterateFromFirst, set, update, get, has, isEqual } from '../functions';
export const DEFAULT_COMPARATOR = function (a, b) {
    return a < b ? -1 : a > b ? 1 : 0;
};
const REDBLACKTREE_TYPE = {
    type: Symbol('Collectable.RedBlackTree'),
    indexable: true,
    equals(other, tree) {
        return isEqual(tree, other);
    },
    unwrap(tree) {
        return unwrap(true, tree);
    },
    get(key, tree) {
        return get(key, tree);
    },
    has(key, tree) {
        return has(key, tree);
    },
    set(key, value, tree) {
        return set(key, value, tree);
    },
    update(key, updater, tree) {
        return update(updater, key, tree);
    },
    verifyKey(key, tree) {
        return isDefined(key);
    }
};
export class RedBlackTreeImpl {
    constructor(_owner, _group, _compare, _root, _size) {
        this._owner = _owner;
        this._group = _group;
        this._compare = _compare;
        this._root = _root;
        this._size = _size;
        this['@@type'] = REDBLACKTREE_TYPE;
    }
    [Symbol.iterator]() {
        return iterateFromFirst(this);
    }
}
export function createTree(mutable, comparator) {
    return new RedBlackTreeImpl(batch.owner(mutable), nextId(), comparator || DEFAULT_COMPARATOR, NONE, 0);
}
export function cloneTree(tree, group, mutable) {
    return new RedBlackTreeImpl(batch.owner(mutable), group, tree._compare, tree._root, tree._size);
}
export function cloneAsMutable(tree) {
    return cloneTree(tree, nextId(), true);
}
export function cloneAsImmutable(tree) {
    return cloneTree(tree, nextId(), false);
}
export function doneMutating(tree) {
    if (tree._owner === -1) {
        tree._owner = 0;
    }
    return tree;
}

//# sourceMappingURL=red-black-tree.js.map
